package com.bergerkiller.bukkit.nolagg.itemstacker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

import net.minecraft.server.IWorldAccess;
import net.minecraft.server.World;
import net.minecraft.server.WorldServer;

import com.bergerkiller.bukkit.common.AsyncTask;
import com.bergerkiller.bukkit.common.SafeField;
import com.bergerkiller.bukkit.common.Task;
import com.bergerkiller.bukkit.common.utils.WorldUtil;

public class StackFormer extends AsyncTask {
	private static StackFormer thread;
	public static SafeField<List<IWorldAccess>> watcherfield;
	private static Task alterRefreshTask;
	private static Task updateTask;
	
	private static Map<WorldServer, WorldStackFormer> globalWorlds = new HashMap<WorldServer, WorldStackFormer>();
    private static List<WorldStackFormer> toAdd = new ArrayList<WorldStackFormer>();
    private static boolean updated = false;
	public static WorldStackFormer get(WorldServer world) {
		WorldStackFormer former = globalWorlds.get(world);
		if (former == null) {
			former = new WorldStackFormer(world);
			globalWorlds.put(world, former);
			synchronized (former) {
				toAdd.add(former);
			}
		}
		return former;
	}
	public static void remove(WorldServer world) {
		WorldStackFormer f = globalWorlds.remove(world);
		if (f != null) {
			f.disable();
		}
	}
	
	public static void init() {
		watcherfield = new SafeField<List<IWorldAccess>>(World.class, "u");
		if (!watcherfield.isValid()) {
			NoLaggItemStacker.plugin.log(Level.WARNING, "Failed to hook into world access listeners, will use a slower alternative!");
			alterRefreshTask = new Task(NoLaggItemStacker.plugin) {
				public void run() {
					for (WorldStackFormer former : globalWorlds.values()) {
						former.watcher.refresh();
					}
				}
			}.start(1, 40);
		}
		
		updateTask = new Task(NoLaggItemStacker.plugin) {
			public void run() {
				for (WorldStackFormer former : globalWorlds.values()) {
					former.update();
				}
				updated = true;
			}
		}.start(20, 20);
		
		for (WorldServer world : WorldUtil.getWorlds()) {
			get(world);
		}
		thread = new StackFormer();
		thread.start(true);
	}
	public static void deinit() {
		Task.stop(updateTask);
		updateTask = null;
		Task.stop(alterRefreshTask);
		alterRefreshTask = null;
		
		AsyncTask.stop(thread);
		thread = null;
		synchronized (toAdd) {
			toAdd.clear();
		}
		for (WorldStackFormer f : globalWorlds.values()) {
			f.disable();
		}
		globalWorlds.clear();
        watcherfield = null;
	}
	
	private final List<WorldStackFormer> worlds;
	
	private StackFormer() {
		synchronized (toAdd) {
			this.worlds = new ArrayList<WorldStackFormer>(toAdd);
			toAdd.clear();
		}
	}
	
	public void run() {
		try {
			if (updated) {
				updated = false;
				synchronized (toAdd) {
					if (!toAdd.isEmpty()) {
						this.worlds.addAll(toAdd);
						toAdd .clear();
					}
				}
				WorldStackFormer f;
				Iterator<WorldStackFormer> iter = this.worlds.iterator();
				while (iter.hasNext()) {
					f = iter.next();
					if (f.isDisabled()) {
						iter.remove();
					} else {
						f.run();
					}
				}
				sleep(300);
			} else {
				sleep(500);
			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}
		
}
